﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2016 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.DataPreprocessing {
  [Item("DateTimePreprocessingDataColumn", "")]
  public class DateTimePreprocessingDataColumn : PreprocessingDataColumn<DateTime> {

    #region Constructor, Cloning & Persistence
    public DateTimePreprocessingDataColumn()
      : base() { }
    public DateTimePreprocessingDataColumn(string name, IEnumerable<DateTime> values)
      : base(name, values) {
    }

    private DateTimePreprocessingDataColumn(DateTimePreprocessingDataColumn original, Cloner cloner)
      : base(original, cloner) {
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new DateTimePreprocessingDataColumn(this, cloner);
    }

    [StorableConstructor]
    private DateTimePreprocessingDataColumn(bool deserializing)
      : base(deserializing) { }
    #endregion

    protected override DateTime DefaultValue { get { return DateTime.MinValue; } }

    #region Statistics
    public TimeSpan GetRange(IEnumerable<int> indices = null) { return GetMax(indices) - GetMin(indices); }
    public DateTime GetMean(IEnumerable<int> indices = null) { return AggregateAsDouble(GetValidValues(indices), Enumerable.Average); }
    public TimeSpan GetStandardDeviation(IEnumerable<int> indices = null) { return AggregateDistanceAsDouble(GetValidValues(indices), EnumerableStatisticExtensions.StandardDeviation); }
    public TimeSpan GetVariance(IEnumerable<int> indices = null) { return AggregateDistanceAsDouble(GetValidValues(indices), EnumerableStatisticExtensions.Variance); }
    #endregion

    #region IStringConvertibleColumn
    public override bool Validate(string value, out string errorMessage) {
      DateTime val;
      bool valid = DateTime.TryParse(value, out val) || string.IsNullOrEmpty(value);
      errorMessage = valid ? string.Empty : "Invalid Value (values must be formatted according to the current culture settings)";
      return valid;
    }
    public override string GetValue(int index) {
      var value = Values[index];
      return IsValidValue(value) ? Values[index].ToString("o") : string.Empty;   // format "s" sortable or "o" roundtrip
    }
    public override bool SetValue(string value, int index) {
      DateTime val;
      if (DateTime.TryParse(value, out val)) {
        Values[index] = val;
        return true;
      } else if (string.IsNullOrEmpty(value)) {
        Values[index] = DateTime.MinValue;
        return true;
      } else {
        return false;
      }
    }
    #endregion

    private static DateTime AggregateAsDouble(IEnumerable<DateTime> values, Func<IEnumerable<double>, double> func) {
      return values.Any() ? new DateTime((long)func(values.Select(x => (double)x.Ticks))) : DateTime.MinValue;
    }
    private static TimeSpan AggregateDistanceAsDouble(IEnumerable<DateTime> values, Func<IEnumerable<double>, double> func) {
      return values.Any() ? new TimeSpan((long)func(values.Select(x => (double)x.Ticks))) : TimeSpan.Zero;
    }
  }
}